*###[ ffxdb1:
	subroutine ffxdb1(cdb1, p, m1, m2, ier)
***#[*comment:***********************************************************
*									*
*	DB1 function (derivative of B1)					*
*									*
*	algorithm adapted from Ansgar Denner's bcanew.f			*
*									*
***#]*comment:*********************************************************** 
*  #[ declarations:
	implicit none
*
*	arguments
*
	DOUBLE COMPLEX cdb1
	DOUBLE PRECISION p, m1, m2
	integer ier

	DOUBLE PRECISION delta
	common /ffcut/ delta

	DOUBLE COMPLEX ffpvf, ffypvf
	external ffpvf, ffypvf

	DOUBLE COMPLEX xp, xm, yp, ym, r

	logical initir
	save initir
	data initir /.FALSE./

	include 'ff.h'

	if(abs(p) .gt. precx*(m1 + m2)) then

*		IR divergent case
	  if(m2 .eq. 0D0 .and. p .eq. m1) then
	    if(.not. initir) then
	      initir = .TRUE.
	      print *, "ffxdb1: IR divergent B1', ",
     +          "using cutoff ", delta
	    endif
	    cdb1 = .5D0*(3D0 + log(delta/p))/p
	    return
	  endif

	  call ffroots(p, m1, m2, xp, xm, yp, ym, r)
	  if(abs(xp - xm) .gt. sqrt(precx)*abs(xp + xm)) then
	    cdb1 = (ffypvf(2, xp, yp) - ffypvf(2, xm, ym))/r
	  else if(abs(xp) .gt. 10D0) then
	    cdb1 = DBLE( (2D0/3D0 +
     +       (2D0 - 3D0*xp)*ffpvf(3, xp, yp))/p/x1**2 )
	  else if(abs(yp) .gt. precx) then
	    cdb1 = DBLE( (3D0/2D0 +
     +       (2D0 - 3D0*xp)*ffpvf(1, xp, yp))/p )
	  else
	    call fferr(101, ier)
	  endif

*		zero momentum case
	else if(abs(m1 - m2) .gt. precx*(m1 + m2)) then
	  xm = DCMPLX(1D0, -1D-5*precx)*m1/(m1 - m2)
	  ym = DCMPLX(1D0, -1D-5*precx)*m2/(m2 - m1)
	  if(abs(x2) .lt. 10D0) then
	    cdb1 = -(1D0/3D0 + ffypvf(2, xm, ym))/(m1 - m2)
	  else
	    cdb1 = -(1D0/3D0 + ffypvf(3, xm, ym))/m1
	  endif

	else
	  cdb1 = -1D0/12D0/m1

	endif

	end


*###[ ffroots
	subroutine ffroots(p, m1, m2, xp, xm, yp, ym, r)
***#[*comment:***********************************************************
*									*
*	roots of quadratic equation					*
*	p*x^2 + (m2 - m1 - p)*x + m2 - eps =				*
*	  p*(x - xp)*(x - xm) = p*(x - 1 + yp)*(x - 1 + ym)		*
*	i.e. x[pm] = 1 - y[pm]						*
*									*
***#]*comment:*********************************************************** 
*  #[ declarations:
	implicit none
*
*	arguments
*
	DOUBLE PRECISION p, m1, m2
	DOUBLE COMPLEX xp, xm, yp, ym, r

	DOUBLE PRECISION q

	include 'ff.h'

	r = sqrt(dcmplx(p*(p - 2D0*(m1 + m2)) + (m1 - m2)**2))
	q = p + m1 - m2
	xp = (q + r)/2D0/p
	xm = (q - r)/2D0/p
	if(abs(xm) .gt. abs(xp)) then
	  xp = m1/p/xm
	else if(abs(xp) .gt. abs(xm)) then
	  xm = m1/p/xp
	endif
	xp = xp + DCMPLX(0D0, abs(p*xp)/p*1D-5*precx)
	xm = xm + DCMPLX(0D0, -abs(p*xm)/p*1D-5*precx)

	q = p - m1 + m2
	ym = (q + r)/2D0/p
	yp = (q - r)/2D0/p
	if(abs(ym) .gt. abs(yp)) then
	  yp = m2/p/ym
	else if(abs(yp) .gt. abs(ym)) then
	  ym = m2/p/yp
	endif
	yp = yp + DCMPLX(0D0, -abs(p*yp)/p*1D-5*precx)
	ym = ym + DCMPLX(0D0, abs(p*ym)/p*1D-5*precx)

	end


*###[ ffpvf
	DOUBLE COMPLEX function ffpvf(n, x, y)
***#[*comment:***********************************************************
*									*
*	Passarino-Veltman function f(n, x)				*
*	here third arg y = 1 - x					*
*									*
***#]*comment:*********************************************************** 
*  #[ declarations:
	implicit none
*
*	arguments
*
	integer n
	DOUBLE COMPLEX x, y

	integer m
	DOUBLE COMPLEX xm

	include 'ff.h'

	if(abs(x) .lt. 10D0) then
	  if(n .eq. 0) then
	    ffpvf = -log(-y/x)
	  else if(x .ne. DCMPLX(0D0)) then
	    ffpvf = 0D0
	    xm = 1D0
	    do 10 m = 0, n - 1
	      ffpvf = ffpvf - xm/(n - m)
	      xm = xm*x
10	    continue
	    ffpvf = ffpvf - xm*log(-y/x)
	  else
	    ffpvf = -1D0/n
	  endif
	else
	  ffpvf = 0D0
	  xm = 1D0
	  do 20 m = 1, 30
	    xm = xm/x
	    ffpvf = ffpvf + xm/(m + n)
	    if(abs(xm/ffpvf) .lt. precx) return
20	  continue
	endif
	end


*###[ ffypvf
	DOUBLE COMPLEX function ffypvf(n, x, y)
***#[*comment:***********************************************************
*									*
*	y*ffpvf(n, x, y)						*
*									*
***#]*comment:*********************************************************** 
*  #[ declarations:
	implicit none
*
*	arguments
*
	integer n
	DOUBLE COMPLEX x, y

	DOUBLE COMPLEX ffpvf
	external ffpvf

	if(abs(y) .eq. 0D0) then
	  ffypvf = 0D0
	else
	  ffypvf = y*ffpvf(n, x, y)
	endif
	end


